"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.InsecureTokenProvider = void 0;
const internal_1 = require("@fluidframework/driver-definitions/internal");
const generateToken_js_1 = require("./generateToken.js");
/**
 * Provides an in memory implementation of {@link @fluidframework/routerlicious-driver#ITokenProvider} that can be
 * used to insecurely connect to the Fluid Relay.
 *
 * As the name implies, this is not secure and should not be used in production.
 * It simply makes examples where authentication is not relevant easier to bootstrap.
 * @internal
 */
class InsecureTokenProvider {
    constructor(
    /**
     * Private server tenantKey for generating tokens.
     */
    tenantKey, 
    /**
     * User with whom generated tokens will be associated.
     */
    user, 
    /**
     * Optional. Override of scopes. If a param is not provided, InsecureTokenProvider
     * will use the default scopes which are document read, write and summarizer write.
     *
     * @param scopes - See {@link @fluidframework/protocol-definitions#ITokenClaims.scopes}
     *
     * @defaultValue [ ScopeType.DocRead, ScopeType.DocWrite, ScopeType.SummaryWrite ]
     */
    scopes) {
        this.tenantKey = tenantKey;
        this.user = user;
        this.scopes = scopes;
    }
    /**
     * {@inheritDoc @fluidframework/routerlicious-driver#ITokenProvider.fetchOrdererToken}
     */
    async fetchOrdererToken(tenantId, documentId) {
        return {
            fromCache: true,
            jwt: (0, generateToken_js_1.generateToken)(tenantId, this.tenantKey, this.scopes ?? [internal_1.ScopeType.DocRead, internal_1.ScopeType.DocWrite, internal_1.ScopeType.SummaryWrite], documentId, this.user),
        };
    }
    /**
     * {@inheritDoc @fluidframework/routerlicious-driver#ITokenProvider.fetchStorageToken}
     */
    async fetchStorageToken(tenantId, documentId) {
        return {
            fromCache: true,
            jwt: (0, generateToken_js_1.generateToken)(tenantId, this.tenantKey, this.scopes ?? [internal_1.ScopeType.DocRead, internal_1.ScopeType.DocWrite, internal_1.ScopeType.SummaryWrite], documentId, this.user),
        };
    }
}
exports.InsecureTokenProvider = InsecureTokenProvider;
//# sourceMappingURL=insecureTokenProvider.js.map