"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.MockSharedObjectServices = exports.MockObjectStorageService = exports.MockEmptyDeltaConnection = exports.MockFluidDataStoreRuntime = exports.MockAudience = exports.MockQuorumClients = exports.MockContainerRuntimeFactory = exports.MockContainerRuntime = exports.MockDeltaConnection = void 0;
const client_utils_1 = require("@fluid-internal/client-utils");
const container_definitions_1 = require("@fluidframework/container-definitions");
const internal_1 = require("@fluidframework/core-utils/internal");
const driver_definitions_1 = require("@fluidframework/driver-definitions");
const internal_2 = require("@fluidframework/driver-definitions/internal");
const internal_3 = require("@fluidframework/runtime-definitions/internal");
const internal_4 = require("@fluidframework/runtime-utils/internal");
const internal_5 = require("@fluidframework/telemetry-utils/internal");
const uuid_1 = require("uuid");
const deepFreeze_js_1 = require("./deepFreeze.js");
const mockDeltas_js_1 = require("./mockDeltas.js");
const mockHandle_js_1 = require("./mockHandle.js");
/**
 * Mock implementation of IDeltaConnection for testing
 * @legacy
 * @alpha
 */
class MockDeltaConnection {
    get connected() {
        return this._connected;
    }
    constructor(submitFn, dirtyFn) {
        this.submitFn = submitFn;
        this.dirtyFn = dirtyFn;
        this._connected = true;
    }
    attach(handler) {
        this.handler = handler;
        handler.setConnectionState(this.connected);
    }
    submit(messageContent, localOpMetadata) {
        return this.submitFn(messageContent, localOpMetadata);
    }
    dirty() {
        this.dirtyFn();
    }
    setConnectionState(connected) {
        this._connected = connected;
        this.handler?.setConnectionState(connected);
    }
    processMessages(messageCollection) {
        this.handler?.processMessages?.(messageCollection);
    }
    reSubmit(content, localOpMetadata, squash) {
        this.handler?.reSubmit(content, localOpMetadata, squash);
    }
    applyStashedOp(content) {
        return this.handler?.applyStashedOp(content);
    }
    rollback(message, localOpMetadata) {
        this.handler?.rollback?.(message, localOpMetadata);
    }
}
exports.MockDeltaConnection = MockDeltaConnection;
const defaultMockContainerRuntimeOptions = {
    flushMode: internal_3.FlushMode.Immediate,
    enableGroupedBatching: false,
};
const makeContainerRuntimeOptions = (mockContainerRuntimeOptions) => ({
    ...defaultMockContainerRuntimeOptions,
    ...mockContainerRuntimeOptions,
});
/**
 * Mock implementation of IContainerRuntime for testing basic submitting and processing of messages.
 * If test specific logic is required, extend this class and add the logic there. For an example, take a look
 * at MockContainerRuntimeForReconnection.
 * @legacy
 * @alpha
 */
class MockContainerRuntime extends client_utils_1.TypedEventEmitter {
    constructor(dataStoreRuntime, factory, mockContainerRuntimeOptions = defaultMockContainerRuntimeOptions, overrides) {
        super();
        this.dataStoreRuntime = dataStoreRuntime;
        this.factory = factory;
        this.overrides = overrides;
        /**
         * @deprecated use the associated datastore to create the delta connection
         */
        this.deltaConnections = [];
        this.pendingMessages = [];
        this.outbox = [];
        this.idAllocationOutbox = [];
        this.deltaManager = new mockDeltas_js_1.MockDeltaManager(() => this.clientId);
        this.deltaManager.inbound.on("push", (message) => {
            this.factory.pushMessage(message);
        });
        const msn = overrides?.minimumSequenceNumber;
        if (msn !== undefined) {
            this.deltaManager.lastSequenceNumber = msn;
            this.deltaManager.minimumSequenceNumber = msn;
        }
        // Set FluidDataStoreRuntime's deltaManager to ours so that they are in sync.
        this.dataStoreRuntime.deltaManagerInternal = this.deltaManager;
        this.dataStoreRuntime.quorum = factory.quorum;
        this.dataStoreRuntime.containerRuntime = this;
        // FluidDataStoreRuntime already creates a clientId, reuse that so they are in sync.
        this.clientId = this.dataStoreRuntime.clientId ?? (0, uuid_1.v4)();
        factory.quorum.addMember(this.clientId, {});
        this.runtimeOptions = makeContainerRuntimeOptions(mockContainerRuntimeOptions);
        (0, internal_1.assert)(this.runtimeOptions.flushMode !== internal_3.FlushMode.Immediate ||
            !this.runtimeOptions.enableGroupedBatching, "Grouped batching is not compatible with FlushMode.Immediate");
    }
    /**
     * @deprecated use the associated datastore to create the delta connection
     */
    createDeltaConnection() {
        const deltaConnection = this.dataStoreRuntime.createDeltaConnection();
        this.deltaConnections.push(deltaConnection);
        return deltaConnection;
    }
    finalizeIdRange(range) {
        (0, internal_1.assert)(this.dataStoreRuntime.idCompressor !== undefined, "Shouldn't try to finalize IdRanges without an IdCompressor");
        this.dataStoreRuntime.idCompressor.finalizeCreationRange(range);
    }
    submit(messageContent, localOpMetadata) {
        const clientSequenceNumber = ++this.deltaManager.clientSequenceNumber;
        const message = {
            content: messageContent,
            localOpMetadata,
            referenceSequenceNumber: this.deltaManager.lastSequenceNumber,
        };
        const isAllocationMessage = this.isAllocationMessage(message.content);
        switch (this.runtimeOptions.flushMode) {
            case internal_3.FlushMode.Immediate: {
                if (!isAllocationMessage) {
                    const idAllocationOp = this.generateIdAllocationOp();
                    if (idAllocationOp !== undefined) {
                        this.submitInternal(idAllocationOp, clientSequenceNumber);
                    }
                }
                this.submitInternal(message, clientSequenceNumber);
                break;
            }
            case internal_3.FlushMode.TurnBased: {
                // Id allocation messages are directly submitted during the resubmit path
                if (isAllocationMessage) {
                    this.idAllocationOutbox.push(message);
                }
                else {
                    this.outbox.push(message);
                }
                break;
            }
            default:
                throw new Error(`Unsupported FlushMode ${this.runtimeOptions.flushMode}`);
        }
        return clientSequenceNumber;
    }
    /**
     * If the message is an idAllocation message, it will finalize the id range and return true.
     * Otherwise, it will return false.
     */
    maybeProcessIdAllocationMessage(message) {
        if (this.isAllocationMessage(message.contents)) {
            this.finalizeIdRange(message.contents.contents);
            return true;
        }
        return false;
    }
    isAllocationMessage(message) {
        return (message !== undefined &&
            message.type === "idAllocation");
    }
    dirty() { }
    get isDirty() {
        return this.pendingMessages.length > 0;
    }
    /**
     * If flush mode is set to FlushMode.TurnBased, it will send all messages queued since the last time
     * this method was called. Otherwise, calling the method does nothing.
     */
    flush() {
        if (this.runtimeOptions.flushMode !== internal_3.FlushMode.TurnBased) {
            return;
        }
        // This mimics the runtime behavior of the IdCompressor by generating an IdAllocationOp
        // and sticking it in front of any op that might rely on that id. It differs slightly in that
        // in the actual runtime it would get put in its own separate batch
        const idAllocationOp = this.generateIdAllocationOp();
        if (idAllocationOp !== undefined) {
            this.idAllocationOutbox.push(idAllocationOp);
        }
        // As with the runtime behavior, we need to send the idAllocationOps first
        const messagesToSubmit = this.idAllocationOutbox.concat(this.outbox);
        this.idAllocationOutbox.length = 0;
        this.outbox.length = 0;
        let fakeClientSequenceNumber = 1;
        messagesToSubmit.forEach((message) => {
            this.submitInternal(message, 
            // When grouped batching is used, the ops within the same grouped batch will have
            // fake sequence numbers when they're ungrouped. The submit function will still
            // return the clientSequenceNumber but this will ensure that the readers will always
            // read the fake client sequence numbers.
            this.runtimeOptions.enableGroupedBatching
                ? fakeClientSequenceNumber++
                : this.deltaManager.clientSequenceNumber);
        });
    }
    /**
     * If flush mode is set to FlushMode.TurnBased, it will rebase the current batch by resubmitting them
     * to the data stores. Otherwise, calling the method does nothing.
     *
     * The method requires `runtimeOptions.enableGroupedBatching` to be enabled.
     */
    rebase() {
        if (this.runtimeOptions.flushMode !== internal_3.FlushMode.TurnBased) {
            return;
        }
        (0, internal_1.assert)(this.runtimeOptions.enableGroupedBatching, "Rebasing is not supported when group batching is disabled");
        // Only outbox needs to be rebased. The idAllocationOutbox is not rebased, as that
        // is a no-op (though resubmitting the other ops may generate new IDs)
        const messagesToRebase = this.outbox.slice();
        this.outbox.length = 0;
        this.reSubmitMessages(messagesToRebase);
    }
    reSubmitMessages(messagesToResubmit) {
        // Sort the messages so that idAllocation messages are submitted first
        // When resubmitting non-idAllocation messages, they may generate new IDs.
        // This sort ensures that all ID ranges are finalized before they are
        // needed (i.e. before the messages that rely on them are processed)
        // and in the order they were allocated
        const orderedMessages = messagesToResubmit
            .filter((message) => message.content.type === "idAllocation")
            .concat(messagesToResubmit.filter((message) => message.content.type !== "idAllocation"));
        orderedMessages.forEach((pendingMessage) => {
            if (pendingMessage.content.type === "idAllocation") {
                this.submit(pendingMessage.content, pendingMessage.localOpMetadata);
            }
            else {
                this.dataStoreRuntime.reSubmit(pendingMessage.content, pendingMessage.localOpMetadata);
            }
        });
    }
    /**
     * Rolls back all pending messages.
     * @remarks
     * This only works when the FlushMode is not immediate as immediate
     * flush mode send the ops to the mock runtime factory for processing/sequencing, and so those
     * ops are no longer local, so not available for rollback.
     */
    rollback() {
        const messagesToRollback = this.outbox.slice().reverse();
        this.outbox.length = 0;
        messagesToRollback.forEach((pm) => {
            this.dataStoreRuntime.rollback?.(pm.content, pm.localOpMetadata);
        });
    }
    generateIdAllocationOp() {
        const idRange = this.dataStoreRuntime.idCompressor?.takeNextCreationRange();
        if (idRange?.ids !== undefined) {
            const allocationOp = {
                type: "idAllocation",
                contents: idRange,
            };
            return {
                content: allocationOp,
                referenceSequenceNumber: this.deltaManager.lastSequenceNumber,
            };
        }
        return undefined;
    }
    submitInternal(message, clientSequenceNumber) {
        // Here, we should instead push to the DeltaManager. And the DeltaManager will push things into the factory's messages
        this.deltaManager.outbound.push([
            {
                clientSequenceNumber,
                contents: message.content,
                referenceSequenceNumber: message.referenceSequenceNumber ?? this.deltaManager.lastSequenceNumber,
                type: internal_2.MessageType.Operation,
            },
        ]);
        this.addPendingMessage(message.content, message.localOpMetadata, clientSequenceNumber);
    }
    process(message) {
        this.deltaManager.process(message);
        const [local, localOpMetadata] = this.processInternal(message);
        if (this.isAllocationMessage(message.contents)) {
            this.finalizeIdRange(message.contents.contents);
        }
        else {
            const messagesContent = [
                {
                    contents: message.contents,
                    clientSequenceNumber: message.clientSequenceNumber,
                    localOpMetadata,
                },
            ];
            this.dataStoreRuntime.processMessages({ envelope: message, local, messagesContent });
        }
    }
    addPendingMessage(content, localOpMetadata, clientSequenceNumber) {
        const pendingMessage = {
            referenceSequenceNumber: this.deltaManager.lastSequenceNumber,
            content,
            clientSequenceNumber,
            localOpMetadata,
        };
        this.pendingMessages.push(pendingMessage);
    }
    processInternal(message) {
        let localOpMetadata;
        const local = this.clientId === message.clientId;
        if (local) {
            const pendingMessage = this.pendingMessages.shift();
            (0, internal_1.assert)(pendingMessage?.clientSequenceNumber === message.clientSequenceNumber, "Unexpected message");
            localOpMetadata = pendingMessage.localOpMetadata;
        }
        return [local, localOpMetadata];
    }
    async resolveHandle(handle) {
        return this.dataStoreRuntime.resolveHandle({
            url: (0, internal_4.toFluidHandleInternal)(handle).absolutePath,
        });
    }
}
exports.MockContainerRuntime = MockContainerRuntime;
/**
 * Factory to create MockContainerRuntime for testing basic submitting and processing of messages.
 * This also acts as a very basic server that stores the messages from all the MockContainerRuntimes and
 * processes them when asked.
 * If test specific logic is required, extend this class and add the logic there. For an example, take a look
 * at MockContainerRuntimeFactoryForReconnection.
 * @legacy
 * @alpha
 */
class MockContainerRuntimeFactory {
    constructor(mockContainerRuntimeOptions = defaultMockContainerRuntimeOptions) {
        this.sequenceNumber = 0;
        this.minSeq = new Map();
        this.quorum = new MockQuorumClients();
        /**
         * The MockContainerRuntimes we produce will push messages into this queue as they are submitted.
         * This is playing the role of the orderer, establishing a single universal order for the messages generated.
         * They are held in this queue until we explicitly choose to process them, at which time they are "broadcast" to
         * each of the runtimes.
         */
        this.messages = [];
        this.runtimes = new Set();
        this.runtimeOptions = makeContainerRuntimeOptions(mockContainerRuntimeOptions);
    }
    get outstandingMessageCount() {
        return this.messages.length;
    }
    /**
     * @returns a minimum sequence number for all connected clients.
     */
    getMinSeq() {
        let minimumSequenceNumber;
        for (const [client, clientSequenceNumber] of this.minSeq) {
            // We have to make sure, a client is part of the quorum, when
            // we compute the msn. We assume that the quorum accurately
            // represents the currently connected clients. In some tests
            // for reconnects, we will remove clients from the quorum
            // to indicate they are currently not connected. In that case,
            // they must no longer contribute to the msn computation.
            if (this.quorum.getMember(client) !== undefined) {
                minimumSequenceNumber =
                    minimumSequenceNumber === undefined
                        ? clientSequenceNumber
                        : Math.min(minimumSequenceNumber, clientSequenceNumber);
            }
        }
        return minimumSequenceNumber ?? 0;
    }
    createContainerRuntime(dataStoreRuntime) {
        const containerRuntime = new MockContainerRuntime(dataStoreRuntime, this, this.runtimeOptions);
        this.runtimes.add(containerRuntime);
        return containerRuntime;
    }
    removeContainerRuntime(containerRuntime) {
        this.runtimes.delete(containerRuntime);
    }
    pushMessage(msg) {
        (0, deepFreeze_js_1.deepFreeze)(msg);
        if (msg.clientId &&
            msg.referenceSequenceNumber !== undefined &&
            !this.minSeq.has(msg.clientId)) {
            this.minSeq.set(msg.clientId, msg.referenceSequenceNumber);
        }
        this.messages.push(msg);
    }
    getFirstMessageToProcess() {
        (0, internal_1.assert)(this.messages.length > 0, "The message queue should not be empty");
        // Explicitly JSON clone the value to match the behavior of going thru the wire.
        const message = JSON.parse(JSON.stringify(this.messages.shift()));
        // TODO: Determine if this needs to be adapted for handling server-generated messages (which have null clientId and referenceSequenceNumber of -1).
        this.minSeq.set(message.clientId, message.referenceSequenceNumber);
        if (this.runtimeOptions.flushMode === internal_3.FlushMode.Immediate ||
            this.lastProcessedMessage?.clientId !== message.clientId) {
            this.sequenceNumber++;
        }
        message.sequenceNumber = this.sequenceNumber;
        message.minimumSequenceNumber = this.getMinSeq();
        this.lastProcessedMessage = message;
        return message;
    }
    processFirstMessage() {
        const message = this.getFirstMessageToProcess();
        for (const runtime of this.runtimes) {
            runtime.process(message);
        }
    }
    /**
     * Process one of the queued messages.  Throws if no messages are queued.
     */
    processOneMessage() {
        if (this.messages.length === 0) {
            throw new Error("Tried to process a message that did not exist");
        }
        this.lastProcessedMessage = undefined;
        this.processFirstMessage();
    }
    /**
     * Process a given number of queued messages.  Throws if there are fewer messages queued than requested.
     * @param count - the number of messages to process
     */
    processSomeMessages(count) {
        if (count > this.messages.length) {
            throw new Error("Tried to process more messages than exist");
        }
        this.lastProcessedMessage = undefined;
        for (let i = 0; i < count; i++) {
            this.processFirstMessage();
        }
    }
    /**
     * Process all remaining messages in the queue.
     */
    processAllMessages() {
        this.lastProcessedMessage = undefined;
        while (this.messages.length > 0) {
            this.processFirstMessage();
        }
    }
}
exports.MockContainerRuntimeFactory = MockContainerRuntimeFactory;
/**
 * @legacy
 * @alpha
 */
class MockQuorumClients {
    constructor(...members) {
        this.eventEmitter = new client_utils_1.EventEmitter();
        this.disposed = false;
        this.members = new Map(members ?? []);
    }
    addMember(id, client) {
        this.members.set(id, client);
        this.eventEmitter.emit("addMember", id, client);
    }
    removeMember(id) {
        if (this.members.delete(id)) {
            this.eventEmitter.emit("removeMember", id);
        }
    }
    getMembers() {
        return this.members;
    }
    getMember(clientId) {
        return this.getMembers().get(clientId);
    }
    dispose() {
        throw new Error("Method not implemented.");
    }
    addListener(event, listener) {
        throw new Error("Method not implemented.");
    }
    on(event, listener) {
        switch (event) {
            case "afterOn":
                this.eventEmitter.on(event, listener);
                return this;
            case "addMember":
            case "removeMember":
                this.eventEmitter.on(event, listener);
                this.eventEmitter.emit("afterOn", event);
                return this;
            default:
                throw new Error("Method not implemented.");
        }
    }
    once(event, listener) {
        throw new Error("Method not implemented.");
    }
    prependListener(event, listener) {
        throw new Error("Method not implemented.");
    }
    prependOnceListener(event, listener) {
        throw new Error("Method not implemented.");
    }
    removeListener(event, listener) {
        this.eventEmitter.removeListener(event, listener);
        return this;
    }
    off(event, listener) {
        this.eventEmitter.off(event, listener);
        return this;
    }
    removeAllListeners(event) {
        throw new Error("Method not implemented.");
    }
    setMaxListeners(n) {
        throw new Error("Method not implemented.");
    }
    getMaxListeners() {
        throw new Error("Method not implemented.");
    }
    listeners(event) {
        throw new Error("Method not implemented.");
    }
    rawListeners(event) {
        throw new Error("Method not implemented.");
    }
    emit(event, ...args) {
        throw new Error("Method not implemented.");
    }
    eventNames() {
        throw new Error("Method not implemented.");
    }
    listenerCount(type) {
        throw new Error("Method not implemented.");
    }
}
exports.MockQuorumClients = MockQuorumClients;
/**
 * @legacy
 * @alpha
 */
class MockAudience extends client_utils_1.TypedEventEmitter {
    constructor() {
        super();
        this.audienceMembers = new Map();
    }
    addMember(clientId, member) {
        this.audienceMembers.set(clientId, member);
        this.emit("addMember", clientId, member);
    }
    removeMember(clientId) {
        const member = this.audienceMembers.get(clientId);
        const deleteResult = this.audienceMembers.delete(clientId);
        this.emit("removeMember", clientId, member);
        return deleteResult;
    }
    getMembers() {
        return new Map(this.audienceMembers.entries());
    }
    getMember(clientId) {
        return this.audienceMembers.get(clientId);
    }
    getSelf() {
        return this._currentClientId === undefined
            ? undefined
            : {
                clientId: this._currentClientId,
            };
    }
    setCurrentClientId(clientId) {
        if (this._currentClientId !== clientId) {
            const oldId = this._currentClientId;
            this._currentClientId = clientId;
            this.emit("selfChanged", oldId === undefined ? undefined : ({ clientId: oldId }), { clientId });
        }
    }
}
exports.MockAudience = MockAudience;
const attachStatesToComparableNumbers = {
    [container_definitions_1.AttachState.Detached]: 0,
    [container_definitions_1.AttachState.Attaching]: 1,
    [container_definitions_1.AttachState.Attached]: 2,
};
/**
 * Mock implementation of IRuntime for testing that does nothing
 * @legacy
 * @alpha
 */
class MockFluidDataStoreRuntime extends client_utils_1.EventEmitter {
    constructor(overrides) {
        super();
        this.readonly = false;
        this.isReadOnly = () => this.readonly;
        this.documentId = undefined;
        this.existing = undefined;
        this.options = {};
        this.path = "";
        this.connected = true;
        this.deltaManagerInternal = new mockDeltas_js_1.MockDeltaManager();
        this.loader = undefined;
        this.quorum = new MockQuorumClients();
        this.audience = new MockAudience();
        this.deltaConnections = [];
        this._disposed = false;
        this.clientId = overrides?.clientId ?? (0, uuid_1.v4)();
        this.entryPoint = (0, internal_4.toFluidHandleInternal)(overrides?.entryPoint ?? new mockHandle_js_1.MockHandle(null, "", ""));
        this.id = overrides?.id ?? (0, uuid_1.v4)();
        const childLoggerProps = {
            namespace: "fluid:MockFluidDataStoreRuntime",
        };
        const logger = overrides?.logger;
        if (logger !== undefined) {
            childLoggerProps.logger = logger;
        }
        this.logger = (0, internal_5.createChildLogger)(childLoggerProps);
        this.idCompressor = overrides?.idCompressor;
        this._attachState = overrides?.attachState ?? container_definitions_1.AttachState.Attached;
        const registry = overrides?.registry;
        if (registry) {
            this.registry = new Map(registry.map((factory) => [factory.type, factory]));
        }
    }
    get IFluidHandleContext() {
        return this;
    }
    get rootRoutingContext() {
        return this;
    }
    get channelsRoutingContext() {
        return this;
    }
    get objectsRoutingContext() {
        return this;
    }
    get deltaManager() {
        return (0, internal_4.toDeltaManagerErased)(this.deltaManagerInternal);
    }
    createDeltaConnection() {
        const deltaConnection = new MockDeltaConnection((messageContent, localOpMetadata) => this.submitMessageInternal(messageContent, localOpMetadata), () => this.setChannelDirty());
        this.deltaConnections.push(deltaConnection);
        return deltaConnection;
    }
    get absolutePath() {
        return `/${this.id}`;
    }
    /**
     * @deprecated Use `attachState` instead
     *
     * @privateRemarks Also remove the setter when this is removed. setters don't get their own doc tags.
     */
    get local() {
        return !this.isAttached;
    }
    set local(local) {
        // this does not validate attach state orders, or fire events to maintain
        // the existing behavior. due to this, this method is deprecated and will
        // be removed
        this._attachState = local ? container_definitions_1.AttachState.Detached : container_definitions_1.AttachState.Attached;
    }
    get disposed() {
        return this._disposed;
    }
    dispose() {
        this._disposed = true;
    }
    async getChannel(id) {
        return null;
    }
    createChannel(id, type) {
        if (this.registry === undefined) {
            // This preserves the behavior of this mock from before registry support was added.
            return null;
        }
        const factory = this.registry.get(type);
        (0, internal_1.assert)(factory !== undefined, "type missing from registry");
        return factory.create(this, id ?? (0, uuid_1.v4)());
    }
    addChannel(channel) { }
    get isAttached() {
        return this.attachState !== container_definitions_1.AttachState.Detached;
    }
    get attachState() {
        return this._attachState;
    }
    get visibilityState() {
        return this.isAttached ? internal_3.VisibilityState.GloballyVisible : internal_3.VisibilityState.NotVisible;
    }
    bindChannel(channel) {
        return;
    }
    attachGraph() {
        return;
    }
    makeVisibleAndAttachGraph() {
        return;
    }
    bind(handle) {
        return;
    }
    getQuorum() {
        return this.quorum;
    }
    getAudience() {
        return this.audience;
    }
    save(message) {
        return;
    }
    async close() {
        return;
    }
    async uploadBlob(blob) {
        return null;
    }
    async getBlob(blobId) {
        return null;
    }
    submitMessage(type, content) {
        return null;
    }
    submitMessageInternal(messageContent, localOpMetadata) {
        (0, internal_1.assert)(this.containerRuntime !== undefined, "The container runtime has not been initialized");
        return this.containerRuntime.submit(messageContent, localOpMetadata);
    }
    setChannelDirty() {
        (0, internal_1.assert)(this.containerRuntime !== undefined, "The container runtime has not been initialized");
        return this.containerRuntime.dirty();
    }
    submitSignal(type, content) {
        return null;
    }
    processMessages(messageCollection) {
        if (this.disposed) {
            return;
        }
        this.deltaConnections.forEach((dc) => {
            dc.processMessages(messageCollection);
        });
    }
    processSignal(message, local) {
        return;
    }
    updateMinSequenceNumber(value) {
        return;
    }
    setConnectionState(connected, clientId) {
        if (connected && clientId !== undefined) {
            this.clientId = clientId;
        }
        this.deltaConnections.forEach((dc) => dc.setConnectionState(connected));
        return;
    }
    notifyReadOnlyState(readonly) {
        this.readonly = readonly;
    }
    async resolveHandle(request) {
        if (request.url !== undefined) {
            return {
                status: 200,
                mimeType: "fluid/object",
                value: request.url,
            };
        }
        return this.request(request);
    }
    async request(request) {
        return null;
    }
    async summarize(fullTree, trackState) {
        const stats = (0, internal_4.mergeStats)();
        stats.treeNodeCount++;
        return {
            summary: {
                type: driver_definitions_1.SummaryType.Tree,
                tree: {},
            },
            stats,
        };
    }
    async getGCData() {
        return {
            gcNodes: {},
        };
    }
    updateUsedRoutes(usedRoutes) { }
    getAttachSnapshot() {
        return [];
    }
    getAttachSummary() {
        const stats = (0, internal_4.mergeStats)();
        stats.treeNodeCount++;
        return {
            summary: {
                type: driver_definitions_1.SummaryType.Tree,
                tree: {},
            },
            stats,
        };
    }
    getAttachGCData(telemetryContext) {
        return {
            gcNodes: {},
        };
    }
    setAttachState(attachState) {
        if (attachState === this._attachState) {
            return;
        }
        const proposedState = attachStatesToComparableNumbers[attachState];
        const startingState = attachStatesToComparableNumbers[this._attachState];
        if (proposedState < startingState) {
            throw new Error(`cannot transition back to ${attachState} from ${this.attachState}`);
        }
        if (startingState < attachStatesToComparableNumbers[container_definitions_1.AttachState.Attaching] &&
            proposedState >= attachStatesToComparableNumbers[container_definitions_1.AttachState.Attaching]) {
            this._attachState = container_definitions_1.AttachState.Attaching;
            this.emit("attaching");
        }
        if (startingState < attachStatesToComparableNumbers[container_definitions_1.AttachState.Attached] &&
            proposedState >= attachStatesToComparableNumbers[container_definitions_1.AttachState.Attached]) {
            this._attachState = container_definitions_1.AttachState.Attached;
            this.emit("attached");
        }
    }
    async waitAttached() {
        return;
    }
    async requestDataStore(request) {
        return null;
    }
    reSubmit(content, localOpMetadata, squash) {
        this.deltaConnections.forEach((dc) => {
            dc.reSubmit(content, localOpMetadata, squash);
        });
    }
    async applyStashedOp(content) {
        return this.deltaConnections.map((dc) => dc.applyStashedOp(content))[0];
    }
    rollback(message, localOpMetadata) {
        this.deltaConnections.forEach((dc) => {
            dc.rollback?.(message, localOpMetadata);
        });
    }
}
exports.MockFluidDataStoreRuntime = MockFluidDataStoreRuntime;
/**
 * Mock implementation of IDeltaConnection
 * @internal
 */
class MockEmptyDeltaConnection {
    constructor() {
        this.connected = false;
    }
    attach(handler) { }
    submit(messageContent) {
        (0, internal_1.assert)(false, "Throw submit error on mock empty delta connection");
    }
    dirty() { }
}
exports.MockEmptyDeltaConnection = MockEmptyDeltaConnection;
/**
 * Mock implementation of IChannelStorageService
 * @legacy
 * @alpha
 */
class MockObjectStorageService {
    constructor(contents) {
        this.contents = contents;
    }
    async readBlob(path) {
        return (0, client_utils_1.stringToBuffer)(this.contents[path], "utf8");
    }
    async contains(path) {
        return this.contents[path] !== undefined;
    }
    async list(path) {
        const pathPartsLength = (0, internal_4.getNormalizedObjectStoragePathParts)(path).length;
        return Object.keys(this.contents).filter((key) => key.startsWith(path) && key.split("/").length === pathPartsLength + 1);
    }
}
exports.MockObjectStorageService = MockObjectStorageService;
/**
 * Mock implementation of IChannelServices
 * @legacy
 * @alpha
 */
class MockSharedObjectServices {
    static createFromSummary(summaryTree) {
        const contents = {};
        setContentsFromSummaryTree(summaryTree, "", contents);
        return new MockSharedObjectServices(contents);
    }
    constructor(contents) {
        this.deltaConnection = new MockEmptyDeltaConnection();
        this.objectStorage = new MockObjectStorageService(contents);
    }
}
exports.MockSharedObjectServices = MockSharedObjectServices;
/**
 * Populate the given `contents` object with all paths/values in a summary tree
 */
function setContentsFromSummaryTree({ tree }, path, contents) {
    for (const [key, value] of Object.entries(tree)) {
        switch (value.type) {
            case driver_definitions_1.SummaryType.Blob:
                (0, internal_1.assert)(typeof value.content === "string", "Unexpected blob value on mock createFromSummary");
                contents[`${path}${key}`] = value.content;
                break;
            case driver_definitions_1.SummaryType.Tree:
                setContentsFromSummaryTree(value, `${path}${key}/`, contents);
                break;
            default:
                (0, internal_1.assert)(false, "Unexpected summary type on mock createFromSummary");
        }
    }
}
//# sourceMappingURL=mocks.js.map