/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { ScopeType } from "@fluidframework/driver-definitions/internal";
import { generateToken } from "./generateToken.js";
/**
 * Provides an in memory implementation of {@link @fluidframework/routerlicious-driver#ITokenProvider} that can be
 * used to insecurely connect to the Fluid Relay.
 *
 * As the name implies, this is not secure and should not be used in production.
 * It simply makes examples where authentication is not relevant easier to bootstrap.
 * @internal
 */
export class InsecureTokenProvider {
    constructor(
    /**
     * Private server tenantKey for generating tokens.
     */
    tenantKey, 
    /**
     * User with whom generated tokens will be associated.
     */
    user, 
    /**
     * Optional. Override of scopes. If a param is not provided, InsecureTokenProvider
     * will use the default scopes which are document read, write and summarizer write.
     *
     * @param scopes - See {@link @fluidframework/protocol-definitions#ITokenClaims.scopes}
     *
     * @defaultValue [ ScopeType.DocRead, ScopeType.DocWrite, ScopeType.SummaryWrite ]
     */
    scopes) {
        this.tenantKey = tenantKey;
        this.user = user;
        this.scopes = scopes;
    }
    /**
     * {@inheritDoc @fluidframework/routerlicious-driver#ITokenProvider.fetchOrdererToken}
     */
    async fetchOrdererToken(tenantId, documentId) {
        return {
            fromCache: true,
            jwt: generateToken(tenantId, this.tenantKey, this.scopes ?? [ScopeType.DocRead, ScopeType.DocWrite, ScopeType.SummaryWrite], documentId, this.user),
        };
    }
    /**
     * {@inheritDoc @fluidframework/routerlicious-driver#ITokenProvider.fetchStorageToken}
     */
    async fetchStorageToken(tenantId, documentId) {
        return {
            fromCache: true,
            jwt: generateToken(tenantId, this.tenantKey, this.scopes ?? [ScopeType.DocRead, ScopeType.DocWrite, ScopeType.SummaryWrite], documentId, this.user),
        };
    }
}
//# sourceMappingURL=insecureTokenProvider.js.map